﻿#region usings
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Windows;
using System.Windows.Controls;

using IdeaBlade.EntityModel;
using IdeaBlade.Core;

using DomainModel;

#endregion

namespace NextSilverlightApp {

  public partial class Page : UserControl {

    #region ctor & load

    public Page() {
      InitializeComponent();
      Loaded += Page_Loaded;
    }

    void Page_Loaded(object sender, RoutedEventArgs e) {
      CreateEntityManager(); // the gateway to persistence
      InitializeQueries();
      Reset();
    }

    private void CreateEntityManager() {
      WriteMessage("Creating EntityManager ...");
      _entityManager = new NorthwindIBEntities(false);
    }


    #endregion

    #region Button Click Handlers

    private void btnConnect_Click(object sender, RoutedEventArgs e) {
      Connect(); // to the server
    }

    private void btnLogin_Click(object sender, RoutedEventArgs e) {
      Login(); // for security and identification
    }

    private void btnFetch_Click(object sender, RoutedEventArgs e) {
      Fetch(); // Get from the database
    }

    private void btnSave_Click(object sender, RoutedEventArgs e) {
      Save();
    }

    private void btnLogout_Click(object sender, RoutedEventArgs e) {
      Logout();
    }

    private void btnReset_Click(object sender, RoutedEventArgs e) {
      Reset();
    }
    #endregion

    #region Initializequeries

    private void InitializeQueries() {
      AddQueries();
      InitializeQueryCombo();
    }

    private void InitializeQueryCombo() {
      var qNames = _queries.Keys.ToList();
      queryCombo.ItemsSource = qNames;
      queryCombo.SelectedItem = qNames.FirstOrDefault();
      queryCombo.SelectionChanged += delegate { Fetch(); };
    }

    #endregion

    #region Fetch

    private void Fetch() {

      var query = GetQuery();
      if (null == query) {
        WriteMessage("No query selected; pick a query.");
        return;
      }

      WriteMessage("Fetching ...");

      _entityManager.ExecuteQueryAsync(
        query,
        args => {
          if (args.HasError) {
            WriteMessage(args.Error.Message);
          } else {
            // Special logic to handle the projection query - 
            //  can't bind to an anonymous type in Silverlight
            if (AnonymousFns.IsAnonymousType(args.EntityQuery.ElementType)) {
              dg.ItemsSource = DynamicTypeConverter.Convert(args.Results);
            } else {
              dg.ItemsSource = args.Results;
            }
            ReportFetchCount(args.Results);
          }
        },
        null);
    }

    private IEntityQuery GetQuery() {
      var queryName = (string)queryCombo.SelectedItem;
      return queryName == null ? null : _queries[queryName];
    }

    private void ReportFetchCount(IEnumerable result) {
      WriteMessage(string.Format("Retrieved {0} customers", result.Cast<object>().Count()));
    }

    #endregion

    #region Queries 

    private void AddQueries() {

      _queries = new Dictionary<string, IEntityQuery>();

      _queries.Add("Get all Customers", _entityManager.Customers);

      _queries.Add("Get Customers starting with A",
                  _entityManager.Customers
                    .Where(c => c.CompanyName.StartsWith("A")));

      _queries.Add("Get Customers with Orders in Oregon",
                  _entityManager.Customers
                    .Where(c => c.Orders.Any(o => o.ShipRegion == "OR")));

      _queries.Add("Get Customers starting with B from cache-only",
                  _entityManager.Customers
                    .Where(c => c.CompanyName.StartsWith("B"))
                    .With(QueryStrategy.CacheOnly));

      _queries.Add("Get Customer info as projection",
                 _entityManager.Customers
                   .Select(c => new { c.CustomerID, c.CompanyName}));


    }

    #endregion

    #region Reset

    private void Reset() {

      // Start all over again ...

      btnLogin.IsEnabled = false;
      btnFetch.IsEnabled = false;
      btnLogout.IsEnabled = false;
      btnSave.IsEnabled = false;
      queryCombo.IsEnabled = false;

      dg.ItemsSource = null;

      _entityManager.Clear();
      if (_entityManager.IsLoggedIn) Logout();
      if (_entityManager.IsConnected) _entityManager.Disconnect();

      WriteMessage("Ready to connect ...");
    }

    #endregion

    #region Connect

    private void Connect() {

      _entityManager.ConnectAsync(
        args => {
          if (args.HasError) {
            WriteMessage(args.Error.Message);
          } else {
            WriteMessage("Connected");
          }
          btnLogin.IsEnabled = args.IsCompleted;
          if (args.CompletedSynchronously) return;
        },
        null);

      if (!_entityManager.IsConnected) WriteMessage("Connecting ...");
    }

    #endregion

    #region Login

    private void Login() {
      // We'll login as a guest.
      LoginCredential cred = null;

      WriteMessage("Logging in ...");

      _entityManager.LoginAsync(
        cred,
        args => {
          if (args.HasError) {
            WriteMessage(args.Error.Message);
          } else {
            WriteMessage("Logged in '" + _entityManager.Principal.Identity.Name + "'");
          }
          queryCombo.IsEnabled = args.IsCompleted;
          btnFetch.IsEnabled = args.IsCompleted;
          btnSave.IsEnabled = args.IsCompleted;
          btnLogout.IsEnabled = args.IsCompleted;
        },
        null);
    }

    #endregion

    #region Logout

    private void Logout() {
      WriteMessage("Logging out ...");

      _entityManager.LogoutAsync(
        args => {
          WriteMessage("Logged out");
          queryCombo.IsEnabled = false;
          btnFetch.IsEnabled = false;
          btnSave.IsEnabled = false;
          btnLogin.IsEnabled = true;
          btnLogout.IsEnabled = false;
        },
        null);
    }

    #endregion

    #region Save

    private void Save() {

      if (!_entityManager.HasChanges()) {
        WriteMessage("No changes to save.");
        return;
      }

      WriteMessage("Saving ...");

      _entityManager.SaveChangesAsync(
        args => {
          if (args.IsCompleted) {
            WriteMessage("Changes saved");
          } else {
            WriteMessage(args.Error.Message);
          }
        },
        null);
    }

    #endregion

    #region Misc

    private void WriteMessage(string msg) {
      txtStatus.Text = msg;
    }

    #endregion

    #region Fields

    private NorthwindIBEntities _entityManager;
    private Dictionary<string, IEntityQuery> _queries;

    #endregion
  }
}
