﻿#Region "usings"
Imports System.Collections
Imports System.Windows
Imports System.Windows.Controls

Imports IdeaBlade.EntityModel
Imports IdeaBlade.Core

Imports DomainModel

#End Region

Namespace NextSilverlightApp

  Partial Public Class Page
    Inherits UserControl

#Region "ctor & load"

    Public Sub New()
      InitializeComponent()
      AddHandler Loaded, AddressOf Page_Loaded
    End Sub

    Private Sub Page_Loaded(ByVal sender As Object, ByVal e As RoutedEventArgs)
      CreateEntityManager() ' the gateway to persistence
      InitializeQueries()
      Reset()
    End Sub

    Private Sub CreateEntityManager()
      WriteMessage("Creating EntityManager ...")
      _entityManager = New NorthwindIBEntities(False)
    End Sub


#End Region

#Region "Button Click Handlers"

    Private Sub btnConnect_Click(ByVal sender As Object, ByVal e As RoutedEventArgs)
      Connect() ' to the server
    End Sub

    Private Sub btnLogin_Click(ByVal sender As Object, ByVal e As RoutedEventArgs)
      Login() ' for security and identification
    End Sub

    Private Sub btnFetch_Click(ByVal sender As Object, ByVal e As RoutedEventArgs)
      Fetch() ' Get from the database
    End Sub

    Private Sub btnSave_Click(ByVal sender As Object, ByVal e As RoutedEventArgs)
      Save()
    End Sub

    Private Sub btnLogout_Click(ByVal sender As Object, ByVal e As RoutedEventArgs)
      Logout()
    End Sub

    Private Sub btnReset_Click(ByVal sender As Object, ByVal e As RoutedEventArgs)
      Reset()
    End Sub
#End Region

#Region "Initializequeries"

    Private Sub InitializeQueries()
      AddQueries()
      InitializeQueryCombo()
    End Sub

    Private Sub InitializeQueryCombo()
      Dim qNames = _queries.Keys.ToList()
      queryCombo.ItemsSource = qNames
      queryCombo.SelectedItem = qNames.FirstOrDefault()
      AddHandler queryCombo.SelectionChanged, Sub() Fetch()
    End Sub

#End Region

#Region "Fetch"

    Private Sub Fetch()

      Dim query = GetQuery()
      If Nothing Is query Then
        WriteMessage("No query selected; pick a query.")
        Return
      End If

      WriteMessage("Fetching ...")

      ' Special logic to handle the projection query - 
      '  can't bind to an anonymous type in Silverlight
      _entityManager.ExecuteQueryAsync(query, Sub(args)
                                                If args.HasError Then
                                                  WriteMessage(args.Error.Message)
                                                Else
                                                  If AnonymousFns.IsAnonymousType(args.EntityQuery.ElementType) Then
                                                    dg.ItemsSource = DynamicTypeConverter.Convert(args.Results)
                                                  Else
                                                    dg.ItemsSource = args.Results
                                                  End If
                                                  ReportFetchCount(args.Results)
                                                End If
                                              End Sub, Nothing)
    End Sub

    Private Function GetQuery() As IEntityQuery
      Dim queryName = CStr(queryCombo.SelectedItem)
      Return If(queryName Is Nothing, Nothing, _queries(queryName))
    End Function

    Private Sub ReportFetchCount(ByVal result As IEnumerable)
      WriteMessage(String.Format("Retrieved {0} customers", result.Cast(Of Object)().Count()))
    End Sub

#End Region

#Region "Queries "

    Private Sub AddQueries()

      _queries = New Dictionary(Of String, IEntityQuery)()

      _queries.Add("Get all Customers", _entityManager.Customers)

      _queries.Add("Get Customers starting with A", _entityManager.Customers.Where(Function(c) c.CompanyName.StartsWith("A")))

      _queries.Add("Get Customers with Orders in Oregon", _entityManager.Customers.Where(Function(c) c.Orders.Any(Function(o) o.ShipRegion = "OR")))

      _queries.Add("Get Customers starting with B from cache-only", _entityManager.Customers.Where(Function(c) c.CompanyName.StartsWith("B")).With(QueryStrategy.CacheOnly))

      _queries.Add("Get Customer info as projection", _entityManager.Customers.Select(Function(c) New With {c.CustomerID, c.CompanyName}))


    End Sub

#End Region

#Region "Reset"

    Private Sub Reset()

      ' Start all over again ...

      btnLogin.IsEnabled = False
      btnFetch.IsEnabled = False
      btnLogout.IsEnabled = False
      btnSave.IsEnabled = False
      queryCombo.IsEnabled = False

      dg.ItemsSource = Nothing

      _entityManager.Clear()
      If _entityManager.IsLoggedIn Then
        Logout()
      End If
      If _entityManager.IsConnected Then
        _entityManager.Disconnect()
      End If

      WriteMessage("Ready to connect ...")
    End Sub

#End Region

#Region "Connect"

    Private Sub Connect()

      _entityManager.ConnectAsync(Sub(args)
                                    If args.HasError Then
                                      WriteMessage(args.Error.Message)
                                    Else
                                      WriteMessage("Connected")
                                    End If
                                    btnLogin.IsEnabled = args.IsCompleted
                                    If args.CompletedSynchronously Then
                                      Return
                                    End If
                                  End Sub, Nothing)

      If Not _entityManager.IsConnected Then
        WriteMessage("Connecting ...")
      End If
    End Sub

#End Region

#Region "Login"

    Private Sub Login()
      ' We'll login as a guest.
      Dim cred As LoginCredential = Nothing

      WriteMessage("Logging in ...")

      _entityManager.LoginAsync(cred, Sub(args)
                                        If args.HasError Then
                                          WriteMessage(args.Error.Message)
                                        Else
                                          WriteMessage("Logged in '" & _entityManager.Principal.Identity.Name & "'")
                                        End If
                                        queryCombo.IsEnabled = args.IsCompleted
                                        btnFetch.IsEnabled = args.IsCompleted
                                        btnSave.IsEnabled = args.IsCompleted
                                        btnLogout.IsEnabled = args.IsCompleted
                                      End Sub, Nothing)
    End Sub

#End Region

#Region "Logout"

    Private Sub Logout()
      WriteMessage("Logging out ...")

      _entityManager.LogoutAsync(Sub(args)
                                   WriteMessage("Logged out")
                                   queryCombo.IsEnabled = False
                                   btnFetch.IsEnabled = False
                                   btnSave.IsEnabled = False
                                   btnLogin.IsEnabled = True
                                   btnLogout.IsEnabled = False
                                 End Sub, Nothing)
    End Sub

#End Region

#Region "Save"

    Private Sub Save()

      If Not _entityManager.HasChanges() Then
        WriteMessage("No changes to save.")
        Return
      End If

      WriteMessage("Saving ...")

      _entityManager.SaveChangesAsync(Sub(args)
                                        If args.IsCompleted Then
                                          WriteMessage("Changes saved")
                                        Else
                                          WriteMessage(args.Error.Message)
                                        End If
                                      End Sub, Nothing)
    End Sub

#End Region

#Region "Misc"

    Private Sub WriteMessage(ByVal msg As String)
      txtStatus.Text = msg
    End Sub

#End Region

#Region "Fields"

    Private _entityManager As NorthwindIBEntities
    Private _queries As Dictionary(Of String, IEntityQuery)

#End Region
  End Class
End Namespace
